@@ -1,18 +1,21 @@
 # Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
-# For details: https://bitbucket.org/ned/coveragepy/src/default/NOTICE.txt
+# For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
 
 """Base test case class for coverage.py testing."""
 
 import contextlib
 import datetime
 import functools
+import glob
 import os
+import os.path
 import random
 import re
 import shlex
 import sys
 import types
 
+import pytest
 from unittest_mixins import (
     EnvironmentAwareMixin, StdStreamCapturingMixin, TempDirMixin,
     DelayedAssertionMixin,
@@ -23,9 +26,9 @@ from coverage import env
 from coverage.backunittest import TestCase, unittest
 from coverage.backward import StringIO, import_local_file, string_class, shlex_quote
 from coverage.cmdline import CoverageScript
-from coverage.debug import _TEST_NAME_FILE
 from coverage.misc import StopEverything
 
+from tests.helpers import arcs_to_arcz_repr, arcz_to_arcs
 from tests.helpers import run_command, SuperModuleCleaner
 
 
@@ -39,24 +42,23 @@ TESTS_DIR = os.path.dirname(__file__)
 def convert_skip_exceptions(method):
     """A decorator for test methods to convert StopEverything to SkipTest."""
     @functools.wraps(method)
-    def wrapper(*args, **kwargs):
-        """Run the test method, and convert exceptions."""
+    def _wrapper(*args, **kwargs):
         try:
             result = method(*args, **kwargs)
         except StopEverything:
             raise unittest.SkipTest("StopEverything!")
         return result
-    return wrapper
+    return _wrapper
 
 
 class SkipConvertingMetaclass(type):
     """Decorate all test methods to convert StopEverything to SkipTest."""
-    def __new__(mcs, name, bases, attrs):
+    def __new__(cls, name, bases, attrs):
         for attr_name, attr_value in attrs.items():
             if attr_name.startswith('test_') and isinstance(attr_value, types.FunctionType):
                 attrs[attr_name] = convert_skip_exceptions(attr_value)
 
-        return super(SkipConvertingMetaclass, mcs).__new__(mcs, name, bases, attrs)
+        return super(SkipConvertingMetaclass, cls).__new__(cls, name, bases, attrs)
 
 
 CoverageTestMethodsMixin = SkipConvertingMetaclass('CoverageTestMethodsMixin', (), {})
@@ -80,6 +82,15 @@ class CoverageTest(
     # Let stderr go to stderr, pytest will capture it for us.
     show_stderr = True
 
+    # Temp dirs go to $TMPDIR/coverage_test/*
+    temp_dir_prefix = "coverage_test/"
+    if os.getenv('COVERAGE_ENV_ID'):
+        temp_dir_prefix += "{}/".format(os.getenv('COVERAGE_ENV_ID'))
+
+    # Keep the temp directories if the env says to.
+    # $set_env.py: COVERAGE_KEEP_TMP - Keep the temp directories made by tests.
+    keep_temp_dir = bool(int(os.getenv("COVERAGE_KEEP_TMP", "0")))
+
     def setUp(self):
         super(CoverageTest, self).setUp()
 
@@ -90,12 +101,6 @@ class CoverageTest(
         self.last_command_output = None
         self.last_module_name = None
 
-        if _TEST_NAME_FILE:                                 # pragma: debugging
-            with open(_TEST_NAME_FILE, "w") as f:
-                f.write("%s_%s" % (
-                    self.__class__.__name__, self._testMethodName,
-                ))
-
     def clean_local_file_imports(self):
         """Clean up the results of calls to `import_local_file`.
 
@@ -129,49 +134,11 @@ class CoverageTest(
         self.last_module_name = 'coverage_test_' + str(random.random())[2:]
         return self.last_module_name
 
-    # Map chars to numbers for arcz_to_arcs
-    _arcz_map = {'.': -1}
-    _arcz_map.update(dict((c, ord(c) - ord('0')) for c in '123456789'))
-    _arcz_map.update(dict(
-        (c, 10 + ord(c) - ord('A')) for c in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
-    ))
-
-    def arcz_to_arcs(self, arcz):
-        """Convert a compact textual representation of arcs to a list of pairs.
-
-        The text has space-separated pairs of letters.  Period is -1, 1-9 are
-        1-9, A-Z are 10 through 36.  The resulting list is sorted regardless of
-        the order of the input pairs.
-
-        ".1 12 2." --> [(-1,1), (1,2), (2,-1)]
-
-        Minus signs can be included in the pairs:
-
-        "-11, 12, 2-5" --> [(-1,1), (1,2), (2,-5)]
-
-        """
-        arcs = []
-        for pair in arcz.split():
-            asgn = bsgn = 1
-            if len(pair) == 2:
-                a, b = pair
-            else:
-                assert len(pair) == 3
-                if pair[0] == '-':
-                    _, a, b = pair
-                    asgn = -1
-                else:
-                    assert pair[1] == '-'
-                    a, _, b = pair
-                    bsgn = -1
-            arcs.append((asgn * self._arcz_map[a], bsgn * self._arcz_map[b]))
-        return sorted(arcs)
-
-    def assert_equal_args(self, a1, a2, msg=None):
+    def assert_equal_arcs(self, a1, a2, msg=None):
         """Assert that the arc lists `a1` and `a2` are equal."""
         # Make them into multi-line strings so we can see what's going wrong.
-        s1 = "\n".join(repr(a) for a in a1) + "\n"
-        s2 = "\n".join(repr(a) for a in a2) + "\n"
+        s1 = arcs_to_arcz_repr(a1)
+        s2 = arcs_to_arcz_repr(a2)
         self.assertMultiLineEqual(s1, s2, msg)
 
     def check_coverage(
@@ -205,11 +172,11 @@ class CoverageTest(
         self.make_file(modname + ".py", text)
 
         if arcs is None and arcz is not None:
-            arcs = self.arcz_to_arcs(arcz)
+            arcs = arcz_to_arcs(arcz)
         if arcs_missing is None:
-            arcs_missing = self.arcz_to_arcs(arcz_missing)
+            arcs_missing = arcz_to_arcs(arcz_missing)
         if arcs_unpredicted is None:
-            arcs_unpredicted = self.arcz_to_arcs(arcz_unpredicted)
+            arcs_unpredicted = arcz_to_arcs(arcz_unpredicted)
 
         # Start up coverage.py.
         cov = coverage.Coverage(branch=True)
@@ -253,19 +220,19 @@ class CoverageTest(
 
         if arcs is not None:
             with self.delayed_assertions():
-                self.assert_equal_args(
-                    analysis.arc_possibilities(), arcs,
-                    "Possible arcs differ: minus is actual, plus is expected"
+                self.assert_equal_arcs(
+                    arcs, analysis.arc_possibilities(),
+                    "Possible arcs differ: minus is expected, plus is actual"
                 )
 
-                self.assert_equal_args(
-                    analysis.arcs_missing(), arcs_missing,
-                    "Missing arcs differ: minus is actual, plus is expected"
+                self.assert_equal_arcs(
+                    arcs_missing, analysis.arcs_missing(),
+                    "Missing arcs differ: minus is expected, plus is actual"
                 )
 
-                self.assert_equal_args(
-                    analysis.arcs_unpredicted(), arcs_unpredicted,
-                    "Unpredicted arcs differ: minus is actual, plus is expected"
+                self.assert_equal_arcs(
+                    arcs_unpredicted, analysis.arcs_unpredicted(),
+                    "Unpredicted arcs differ: minus is expected, plus is actual"
                 )
 
         if report:
@@ -294,8 +261,9 @@ class CoverageTest(
 
         """
         saved_warnings = []
-        def capture_warning(msg, slug=None):
+        def capture_warning(msg, slug=None, once=False):        # pylint: disable=unused-argument
             """A fake implementation of Coverage._warn, to capture warnings."""
+            # NOTE: we don't implement `once`.
             if slug:
                 msg = "%s (%s)" % (msg, slug)
             saved_warnings.append(msg)
@@ -305,7 +273,7 @@ class CoverageTest(
 
         try:
             yield
-        except:
+        except:                     # pylint: disable=try-except-raise
             raise
         else:
             if warnings:
@@ -347,6 +315,13 @@ class CoverageTest(
         msg = "File %r shouldn't exist" % fname
         self.assertTrue(not os.path.exists(fname), msg)
 
+    def assert_file_count(self, pattern, count):
+        """Assert that there are `count` files matching `pattern`."""
+        files = sorted(glob.glob(pattern))
+        msg = "There should be {} files matching {!r}, but there are these: {}"
+        msg = msg.format(count, pattern, files)
+        self.assertEqual(len(files), count, msg)
+
     def assert_starts_with(self, s, prefix, msg=None):
         """Assert that `s` starts with `prefix`."""
         if not s.startswith(prefix):
@@ -355,12 +330,10 @@ class CoverageTest(
     def assert_recent_datetime(self, dt, seconds=10, msg=None):
         """Assert that `dt` marks a time at most `seconds` seconds ago."""
         age = datetime.datetime.now() - dt
-        # Python2.6 doesn't have total_seconds :(
-        self.assertEqual(age.days, 0, msg)
-        self.assertGreaterEqual(age.seconds, 0, msg)
-        self.assertLessEqual(age.seconds, seconds, msg)
+        self.assertGreaterEqual(age.total_seconds(), 0, msg)
+        self.assertLessEqual(age.total_seconds(), seconds, msg)
 
-    def command_line(self, args, ret=OK, _covpkg=None):
+    def command_line(self, args, ret=OK):
         """Run `args` through the command line.
 
         Use this when you want to run the full coverage machinery, but in the
@@ -372,7 +345,7 @@ class CoverageTest(
         Returns None.
 
         """
-        ret_actual = command_line(args, _covpkg=_covpkg)
+        ret_actual = command_line(args)
         self.assertEqual(ret_actual, ret)
 
     coverage_command = "coverage"
@@ -407,7 +380,7 @@ class CoverageTest(
             Python interpreter.
 
         * "coverage" is replaced with the command name for the main
-            Coverage.py program.
+            coverage.py program.
 
         Returns a pair: the process' exit status and its stdout/stderr text,
         which are also stored as `self.last_command_status` and
@@ -438,8 +411,8 @@ class CoverageTest(
                 # modules.
                 command_words = "jython -m coverage".split()
             else:
-                # The invocation requests the Coverage.py program.  Substitute the
-                # actual Coverage.py main command name.
+                # The invocation requests the coverage.py program.  Substitute the
+                # actual coverage.py main command name.
                 command_words = [self.coverage_command]
 
         else:
@@ -461,13 +434,6 @@ class CoverageTest(
         pypath += testmods + os.pathsep + zipfile
         self.set_environ(pythonpath_name, pypath)
 
-        # There are environment variables that we set when we are running the
-        # coverage test suite under coverage.  We don't want these environment
-        # variables to leak into subprocesses we start for a test. Delete them
-        # before running the subprocess command.
-        self.del_environ("COVERAGE_COVERAGE")
-        self.del_environ("COVERAGE_PROCESS_START")
-
         self.last_command_status, self.last_command_output = run_command(cmd)
         print(self.last_command_output)
         return self.last_command_status, self.last_command_output
@@ -501,6 +467,15 @@ class CoverageTest(
         """Return the last line of `report` with the spaces squeezed down."""
         return self.squeezed_lines(report)[-1]
 
+    def get_measured_filenames(self, coverage_data):
+        """Get paths to measured files.
+
+        Returns a dict of {filename: absolute path to file}
+        for given CoverageData.
+        """
+        return {os.path.basename(filename): filename
+                for filename in coverage_data.measured_files()}
+
 
 class UsingModulesMixin(object):
     """A mixin for importing modules from tests/modules and tests/moremodules."""
@@ -513,14 +488,17 @@ class UsingModulesMixin(object):
         sys.path.append(self.nice_file(TESTS_DIR, 'moremodules'))
 
 
-def command_line(args, **kwargs):
+def command_line(args):
     """Run `args` through the CoverageScript command line.
 
-    `kwargs` are the keyword arguments to the CoverageScript constructor.
-
     Returns the return code from CoverageScript.command_line.
 
     """
-    script = CoverageScript(**kwargs)
+    script = CoverageScript()
     ret = script.command_line(shlex.split(args))
     return ret
+
+
+def xfail(condition, reason):
+    """A decorator to mark as test as expected to fail."""
+    return pytest.mark.xfail(condition, reason=reason, strict=True)
